@@ -1,23 +1,22 @@
 package HTTP::BrowserDetect;
 
 use strict;
-use vars qw($VERSION $REVISION @ISA @EXPORT @EXPORT_OK @ALL_TESTS);
+use vars qw($VERSION @ISA @EXPORT @EXPORT_OK @ALL_TESTS);
 require Exporter;
 
 @ISA	   = qw(Exporter);
 @EXPORT	   = qw();
 @EXPORT_OK = qw();
-$REVISION  = '$Id: BrowserDetect.pm,v 1.1 1999/03/17 04:38:06 lee Exp lee $';
-$VERSION   = '0.96';
+$VERSION   = '0.99';
 
 # Operating Systems
-push @ALL_TESTS,(qw(win16 win3x win31 win95 win98 winnt windows win32 win2k mac mac68k macppc os2 unix sun sun4 sun5 suni86 irix irix5 irix6 hpux hpux9 hpux10 aix aix1 aix2 aix3 aix4 linux sco unixware mpras reliant dec sinix freebsd bsd vms x11 amiga));
+push @ALL_TESTS,(qw(win16 win3x win31 win95 win98 winnt windows win32 win2k winxp win2k3 winvista winme dotnet mac macosx mac68k macppc os2 unix sun sun4 sun5 suni86 irix irix5 irix6 hpux hpux9 hpux10 aix aix1 aix2 aix3 aix4 linux sco unixware mpras reliant dec sinix freebsd bsd vms x11 amiga));
 
 # Devices
-push @ALL_TESTS,(qw(palm audrey iopener wap));
+push @ALL_TESTS,(qw(palm audrey iopener wap blackberry));
 
 # Browsers
-push @ALL_TESTS,(qw(mosaic netscape nav2 nav3 nav4 nav4up nav45 nav5 navgold ie ie3 ie4 ie4up ie5 ie55 opera lynx aol aol3 neoplanet neoplanet2 avantgo emacs));
+push @ALL_TESTS,(qw(mosaic netscape nav2 nav3 nav4 nav4up nav45 nav45up nav6 nav6up navgold firefox safari ie ie3 ie4 ie4up ie5 ie5up ie55 ie55up ie6 opera opera3 opera4 opera5 opera6 opera7 lynx links aol aol3 aol4 aol5 aol6 neoplanet neoplanet2 avantgo emacs mozilla gecko));
 
 # Robots
 push @ALL_TESTS,(qw(wget getright robot yahoo altavista lycos infoseek lwp webcrawler linkexchange slurp webtv staroffice lotusnotes konqueror icab google java));
@@ -83,7 +82,22 @@ sub _test {
                                           [\d\.]*		# Throw away remaining numbers and dots
                                           ( [^\s]* )		# Beta version string is up to next space
                                         /x);
-
+     
+  # Firefox version
+  
+  if ($ua =~ /(firefox|firebird|phoenix)/i) {
+      (undef,$major, $minor)    = ($ua =~ / 
+				    (firefox|firebird|phoenix)
+				    \/
+				    ( [^\.]* )			# Major version number is everything before first dot
+				    \.			    	# The first dot
+				    ( [\d]* )			# Minor version nnumber is digits after first dot
+				    /x);                    
+  }
+ 
+  
+  
+  # IE version                               
   if (index($ua,"compatible") !=-1) {
       ($major, $minor, $beta)    = ($ua =~ / 
 				    compatible;       
@@ -99,50 +113,116 @@ sub _test {
 				    ;
 				    /x);
 
-
   }
 
+  $minor = 0+".$minor"; 
+
   $self->{tests} = {};
   my $tests = $self->{tests};
 
-  # Netscape browsers
+  # Mozilla browsers 
   
-  $tests->{NETSCAPE}  = (index($ua,"mozilla") != -1 && index($ua,"spoofer")==-1 && index($ua,"compatible")==-1);
-  $tests->{NAV2}      = (($tests->{NETSCAPE}) && $major == 2);
-  $tests->{NAV3}      = (($tests->{NETSCAPE}) && $major == 3);
-  $tests->{NAV4}      = (($tests->{NETSCAPE}) && $major == 4);
-  $tests->{NAV45}     = (($tests->{NETSCAPE}) && $major == 4 && $minor == 5);
-  $tests->{NAV4UP}    = (($tests->{NETSCAPE}) && $major >= 4);   
-  $tests->{NAVGOLD}   = (index($beta,"gold") != -1);
-  $tests->{NAV5}      = (($tests->{NETSCAPE}) && $major == 5);
+  $tests->{GECKO}     = (index($ua,"gecko") != -1);
+  $tests->{FIREFOX}   = (index($ua,"firefox") != -1) ||
+                        (index($ua,"firebird") != -1) ||
+                        (index($ua,"phoenix") != -1);
+                        
+  $tests->{SAFARI}    = (index($ua,"safari") != -1) || (index($ua,"applewebkit") != -1);
+  
+  # Safari Version
+  if ($tests->{SAFARI}) {
+      my ($safari_build, $safari_minor);
+      ($safari_build,$safari_minor)    = ($ua =~ / 
+				    safari
+				    \/
+				    ( [^\.]* )			# Major version number is everything before first dot
+				    (?:			    	# The first dot
+				    ( [\d]* ))?			# Minor version nnumber is digits after first dot
+				    /x);                    
+    $major = int($safari_build / 100);
+    $minor = int($safari_build % 100) / 100;
+    $beta = $safari_minor;    
+    #print "major=$major minor=$minor beta=$beta\n";
+    
+  }
+  
+ # Gecko-powered Netscape (i.e. Mozilla) versions
+ $tests->{NETSCAPE}  = (!$tests->{FIREFOX} && !$tests->{SAFARI} && index($ua,"mozilla") != -1 && 
+			 index($ua,"spoofer") == -1 && index($ua,"compatible") == -1 &&
+			 index($ua,"opera") == -1 && index($ua,"webtv") == -1 && 
+			 index($ua,"hotjava") == -1);
+ 
+ 
+  if ($tests->{GECKO} && $tests->{NETSCAPE} && index($ua,"netscape")!=-1) {
+      ($major, $minor, $beta)    = ($ua =~ / 
+				    netscape\/  
+				    ( [^\.]* )			# Major version number is everything before first dot
+				    \.				# The first dot
+				    ( [\d]* )			# Minor version nnumber is digits after first dot
+                    ( [^\s]* )
+                   /x);
+     $minor = 0+".$minor"; 
+    #print "major=$major minor=$minor beta=$beta\n";
+  }
 
+  
+  # Netscape browsers
+  $tests->{NAV2}      = ($tests->{NETSCAPE} && $major == 2);
+  $tests->{NAV3}      = ($tests->{NETSCAPE} && $major == 3);
+  $tests->{NAV4}      = ($tests->{NETSCAPE} && $major == 4);
+  $tests->{NAV4UP}    = ($tests->{NETSCAPE} && $major >= 4); 
+  $tests->{NAV45}     = ($tests->{NETSCAPE} && $major == 4 && $minor == .5);
+  $tests->{NAV45UP}   = ($tests->{NAV4} && $minor >= .5) || ($tests->{NETSCAPE} && $major >= 5);
+  $tests->{NAVGOLD}   = (index($beta,"gold") != -1);
+  $tests->{NAV6}      = ($tests->{NETSCAPE} && $major == 5); # go figure
+  $tests->{NAV6UP}    = ($tests->{NETSCAPE} && $major >= 5);
+  
+  $tests->{MOZILLA}   = ($tests->{NETSCAPE} && $tests->{GECKO});
+    
   # Internet Explorer browsers
 
   $tests->{IE}               = (index($ua,"msie") != -1 || index($ua,'microsoft internet explorer') != -1);  
-  $tests->{IE3}              = (($tests->{IE}) && $major == 3);
-  $tests->{IE4}              = (($tests->{IE}) && $major == 4);
-  $tests->{IE4UP}            = (($tests->{IE}) && $major >= 4);
-  $tests->{IE5}              = (($tests->{IE}) && $major == 5);
-  $tests->{IE55}             = (($tests->{IE}) && $major == 5 && $minor == 5);
-
+  $tests->{IE3}              = ($tests->{IE} && $major == 3);
+  $tests->{IE4}              = ($tests->{IE} && $major == 4);
+  $tests->{IE4UP}            = ($tests->{IE} && $major >= 4);
+  $tests->{IE5}              = ($tests->{IE} && $major == 5);
+  $tests->{IE5UP}            = ($tests->{IE} && $major >= 5);
+  $tests->{IE55}             = ($tests->{IE} && $major == 5 && $minor >= .5);
+  $tests->{IE55UP}           = ($tests->{IE5} && $minor >= .5) || ($tests->{IE} && $major >= 6);
+  $tests->{IE6}             =  ($tests->{IE} && $major == 6);
+ 
   # Neoplanet browsers
 
   $tests->{NEOPLANET}      = (index($ua,"neoplanet") != -1);
   $tests->{NEOPLANET2}     = ($tests->{NEOPLANET} && index($ua,"2.") != -1); 
 
+  # AOL Browsers
+
+  $tests->{AOL}       = (index($ua,"aol")     != -1);
+  $tests->{AOL3}      = (index($ua,"aol 3.0") != -1) || ($tests->{AOL} && $tests->{IE3});
+  $tests->{AOL4}      = (index($ua,"aol 4.0") != -1) || ($tests->{AOL} && $tests->{IE4});
+  $tests->{AOL5}      = (index($ua,"aol 5.0") != -1);
+  $tests->{AOL6}      = (index($ua,"aol 6.0") != -1);
+  $tests->{AOLTV}     = (index($ua,"navio") != -1) || (index($ua,"navio_aoltv") != -1);
+
   # Opera browsers
   
-  $tests->{OPERA} = (index($ua,"opera") != -1);
+  $tests->{OPERA}     = (index($ua,"opera")   != -1);
+  $tests->{OPERA3}    = (index($ua,"opera 3") != -1) || (index($ua,"opera/3") != -1);
+  $tests->{OPERA4}    = (index($ua,"opera 4") != -1) || (index($ua,"opera/4") != -1);
+  $tests->{OPERA5}    = (index($ua,"opera 5") != -1) || (index($ua,"opera/5") != -1);
+  $tests->{OPERA6}    = (index($ua,"opera 6") != -1) || (index($ua,"opera/6") != -1);
+  $tests->{OPERA7}    = (index($ua,"opera 7") != -1) || (index($ua,"opera/7") != -1);
   
   # Other browsers
 
+  $tests->{CURL}           = (index($ua,"libcurl") != -1);
   $tests->{STAROFFICE}     = (index($ua,"staroffice") != -1);
   $tests->{ICAB}           = (index($ua,"icab") != -1);
   $tests->{LOTUSNOTES}     = (index($ua,"lotus-notes") != -1);
   $tests->{KONQUEROR}      = (index($ua,"konqueror") != -1);
   $tests->{LYNX}           = (index($ua,"lynx") != -1);
-  $tests->{AOL}            = (index($ua,"aol") != -1);
-  $tests->{AOL3}           = (index($ua,"aol 3.0") != -1);
+  $tests->{LINKS}          = (index($ua,"links") != -1);
   $tests->{WEBTV}          = (index($ua,"webtv") != -1);
   $tests->{MOSAIC}         = (index($ua,"mosaic") != -1);
   $tests->{WGET}           = (index($ua,"wget") != -1);
@@ -182,10 +262,13 @@ sub _test {
 			      index($ua,"find") != -1 ||
 			      index($ua,"index") != -1 ||
 			      index($ua,"copy") != -1 ||
-			      index($ua,"fetch") != -1);
+			      index($ua,"fetch") != -1 ||
+                  index($ua,"ia_archive") != -1 ||
+                  index($ua,"zyborg") != -1);
 
   # Devices
 
+  $tests->{BLACKBERRY}        = (index($ua,"blackberry") != -1);
   $tests->{AUDREY}        = (index($ua,"audrey") != -1);
   $tests->{IOPENER}       = (index($ua,"i-opener") != -1);
   $tests->{AVANTGO}        = (index($ua,"avantgo") != -1);
@@ -215,21 +298,33 @@ sub _test {
   $tests->{WIN31}    = (index($ua,"win16") != -1 || index($ua,"windows 3.1") != -1 || index($ua,"windows 16-bit") != -1);
   $tests->{WIN95}    = (index($ua,"win95") != -1 || index($ua,"windows 95") != -1);
   $tests->{WIN98}    = (index($ua,"win98") != -1 || index($ua,"windows 98") != -1);
-  $tests->{WINNT}    = (index($ua,"winnt") != -1 || index($ua,"windows nt") != -1);
-  $tests->{WIN2K}    = (index($ua,"nt 5") != -1);
-  $tests->{WIN32}    = (($tests->{WIN95} || $tests->{WIN98} || $tests->{WINNT} || $tests->{WIN2K}) || index($ua,"win32") != -1);
+  $tests->{WINNT}    = (index($ua,"winnt") != -1 || 
+			index($ua,"windows nt") != -1 || 
+			index($ua,"nt4") != -1 || 
+			index($ua,"nt3") != -1);
+  $tests->{WIN2K}    = (index($ua,"nt 5.0") != -1 || index($ua,"nt5") != -1);
+  $tests->{WINXP}    = (index($ua,"nt 5.1") != -1 );
+  $tests->{WIN2K3}   = (index($ua,"nt 5.2") != -1 );
+  $tests->{WINVISTA} = (index($ua,"nt 6.0") != -1 );
+  $tests->{DOTNET}   = (index($ua,".net clr") != -1);
+  
+  $tests->{WINME}    = (index($ua,"win 9x 4.90") != -1); # whatever
+  $tests->{WIN32}    = (($tests->{WIN95} || $tests->{WIN98} || $tests->{WINME} || $tests->{WINNT} || 
+			 $tests->{WIN2K}) || $tests->{WINXP} ||  $tests->{WIN2K3} || $tests->{WINVISTA} || index($ua,"win32") != -1);
   $tests->{WINDOWS}  = (($tests->{WIN16} || $tests->{WIN31} || $tests->{WIN95} || $tests->{WIN98} ||
-			 $tests->{WINNT} || $tests->{WIN32} || $tests->{WIN2K}) || index($ua,"win") != -1);
+			 $tests->{WINNT} || $tests->{WIN32} || $tests->{WIN2K} || $tests->{WINXP} || $tests->{WIN2K3} || $tests->{WINVISTA} || $tests->{WINME}) || index($ua,"win") != -1);
   
   # Mac operating systems
   
   $tests->{MAC}      = (index($ua,"macintosh") != -1 || index($ua,"mac_") != -1);
+  $tests->{MACOSX}   = (index($ua,"macintosh") != -1 && index($ua,"mac os x") != -1);
   $tests->{MAC68K}   = (($tests->{MAC}) && (index($ua,"68k") != -1 || index($ua,"68000") != -1));
   $tests->{MACPPC}   = (($tests->{MAC}) && (index($ua,"ppc") != -1 || index($ua,"powerpc") != -1));
   
   # Others
   
   $tests->{AMIGA}   = (index($ua,'amiga') != -1);
+  
   $tests->{EMACS}   = (index($ua,'emacs') != -1);
   $tests->{OS2}     = (index($ua,'os/2') != -1);
   
@@ -272,11 +367,27 @@ sub _test {
 			$tests->{BSD}); 
 
   $tests->{VMS} = (index($ua,"vax") != -1 || index($ua,"openvms") != -1);
-  
-  $self->{major}  = $major;
-  $self->{minor}  = $minor;
-  $self->{beta}	  = $beta;
 
+  # A final try at browser version, if we haven't gotten it so far
+  if ($major eq '') {
+     if ($ua =~ /[A-Za-z]+\/(\d+)\;/) {
+          $major = $1;
+          $minor = 0;
+      }
+  }
+
+  
+  # Gecko version
+  $self->{gecko_version} = undef;
+  if ($tests->{GECKO}) {
+    if( $ua =~ /\([^)]*rv:([\w\.\d]*)/ ) {
+        $self->{gecko_version} = $1;
+    }
+  }
+  
+  $self->{major} = $major;
+  $self->{minor} = $minor;
+  $self->{beta} = $beta;
 }
     
 sub browser_string {
@@ -285,6 +396,8 @@ sub browser_string {
     my $user_agent = $self->user_agent;
     if (defined $user_agent) {
         $browser_string = 'Netscape' if $self->netscape;
+        $browser_string = 'Firefox' if $self->firefox;
+        $browser_string = 'Safari' if $self->safari;
         $browser_string = 'MSIE' if $self->ie;  
         $browser_string = 'WebTV' if $self->webtv;
         $browser_string = 'AOL Browser' if $self->aol;
@@ -303,7 +416,12 @@ sub os_string {
         $os_string = 'Win95' if $self->win95;
         $os_string = 'Win98' if $self->win98;
         $os_string = 'WinNT' if $self->winnt;
+        $os_string = 'Win2k' if $self->win2k;
+        $os_string = 'WinXP' if $self->winxp;
+        $os_string = 'Win2k3' if $self->win2k3;
+        $os_string = 'WinVista' if $self->winvista;
         $os_string = 'Mac' if $self->mac;
+        $os_string = 'Mac OS X' if $self->macosx;
         $os_string = 'Win3x' if $self->win3x;
         $os_string = 'OS2' if $self->os2;
         $os_string = 'Unix' if $self->unix && !$self->linux;
@@ -311,16 +429,23 @@ sub os_string {
     }
     return $os_string;
 }
+
+sub gecko_version {
+  my ($self, $check) = _self_or_default(@_);
+  my $version;
+  $version = $self->{gecko_version};
+  if (defined $check) { 
+    return $check == $version;
+  } else {
+    return $version;
+  }
+}
+
     
 sub version {
   my ($self, $check) = _self_or_default(@_);
   my $version;
-  if (defined $self->{major} && defined $self->{minor}) {
-    $version = join '.' , $self->{major}, $self->{minor};
-    $version += 0;
-  } else {
-    $version = $self->{major};
-  }
+  $version = $self->{major} + $self->{minor};
   if (defined $check) { 
     return $check == $version;
   } else {
@@ -342,7 +467,7 @@ sub minor {
   my ($self, $check) = _self_or_default(@_);
   my ($version) = $self->{minor};
   if (defined $check) { 
-    return ".$check" == ".$version";
+      return ($check == $self->{minor});
   } else {
     return $version;
   }
@@ -365,33 +490,37 @@ __END__
 
 =head1 NAME
 
-HTTP::BrowserDetect - Determine the Web browser, version, and platform
-from an HTTP user agent string
+HTTP::BrowserDetect - Determine the Web browser, version, and platform from an HTTP user agent string
 
 =head1 SYNOPSIS
 
     use HTTP::BrowserDetect;
-      
+
     my $browser = new HTTP::BrowserDetect($user_agent_string);
+
+    # Detect operating system
     if ($browser->windows) {
       if ($browser->winnt) ...
       if ($brorwser->win95) ...
     }
-
-    print $browser->netscape;
-    if (browser->major(4)) {
-      ...;
-    }
-    if (browser->minor(5)) {
-      ...;
-    }
-    
-    $browser->user_agent($another_user_agent_string);
     print $browser->mac;
+
+    # Detect browser vendor and version
+    print $browser->netscape;
     print $browser->ie;
+    if (browser->major(4)) {
+	if ($browser->minor() > .5) {
+	    ...
+	}
+    }
     if ($browser->version() > 4) {
       ...;
     }
+    
+    # Process a different user agent string
+    $browser->user_agent($another_user_agent_string);
+
+
 
 =head1 DESCRIPTION
 
@@ -401,7 +530,7 @@ the object.
 
 This module is based upon the JavaScript browser detection code
 available at
-B<http://developer.netscape.com/docs/examples/javascript/browser_type.html>.
+B<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>.
 
 =head2 CREATING A NEW BROWSER DETECT OBJECT AND SETTING THE USER AGENT STRING
 
@@ -434,23 +563,27 @@ each time.
 
 =item major($major)
 
-Returns the portion of the browser version up to the first decimal
-point as a string.  If passed a parameter, returns true if it equals
-the major version specified by the user agent string.
+Returns the integer portion of the browser version.
+If passed a parameter, returns true if it equals
+the browser major version.
 
 =item minor($minor)
 
-Returns the numeric portion of the browser after first decimal point
-as a string.  If passed a parameter, returns true if it equals the
-minor version specified by the user agent string.  On occasion a
-version may have more than one decimal point, such as 'Wget/1.4.5'.
-The minor version does not include the second decimal point, or any
-further digits or decimals.
+Returns the decimal portion of the browser version as a B<floating-point number> less than 1.  
+For example, if the version is 4.05, this method returns .05; if the version is 4.5, this method returns .5.  
+B<This is a change in behavior from previous versions of this module, which returned a 
+string>.
+
+If passed a parameter, returns true if equals the minor version.  
+
+On occasion a version may have more than one decimal point, such 
+as 'Wget/1.4.5'. The minor version does not include the second decimal point, 
+or any further digits or decimals.
 
 =item version($version)
 
 Returns the version as a floating-point number.  If passed a
-parameter, returns true if it is numerically equal to the version
+parameter, returns true if it is equal to the version
 specified by the user agent string.
 
 =item beta($beta)
@@ -458,33 +591,51 @@ specified by the user agent string.
 Returns any the beta version, consisting of any non-numeric characters
 after the version number.  For instance, if the user agent string is
 'Mozilla/4.0 (compatible; MSIE 5.0b2; Windows NT)', returns 'b2'.  If
-passed a parameter, returns true if equal to the beta version.
+passed a parameter, returns true if equal to the beta version.  If the beta
+starts with a dot, it is thrown away.
 
 =back
 
 =head2 DETECTING OS PLATFORM AND VERSION
 
 The following methods are available, each returning a true or false
-value.  Some methods also test for the operating system version.
+value.  Some methods also test for the operating system version.  
+The indentations below show the hierarchy of tests (for example, win2k 
+is considered a type of winnt, which is a type of win32)
 
-  windows win16 win3x win31 win95 win98 winnt win32 win2k
-  mac mac68k macppc
+  windows 
+    win16 win3x win31  
+    win32 
+        winme win95 win98
+        winnt
+            win2k winxp win2k3 winvista
+  dotnet
+  
+  mac 
+    mac68k macppc macosx
+  
   os2
+  
   unix 
-  sun sun4 sun5 suni86 irix irix5 irix6 hpux hpux9 hpux10 
-  aix aix1 aix2 aix3 aix4 linux sco unixware mpras reliant 
-  dec sinix freebsd bsd
+    sun sun4 sun5 suni86 irix irix5 irix6 hpux hpux9 hpux10 
+    aix aix1 aix2 aix3 aix4 linux sco unixware mpras reliant 
+    dec sinix freebsd bsd
+  
   vms
+
   amiga
 
+It may not be possibile to detect Win98 in Netscape 4.x and earlier. 
+On Opera 3.0, the userAgent string includes "Windows 95/NT4" on all Win32, so you can't distinguish between Win95 and WinNT.
+
 =over 
 
 =item os_string()
 
-Returns one of the following strings, or undef.  These are the same set of strings returned by the
+Returns one of the following strings, or undef.  This method exists solely for compatibility with the
 B<HTTP::Headers::UserAgent> module.
 
-  Win95, Win98, WinNT, Mac, Win3x, OS2, Unix, Linux
+  Win95, Win98, WinNT, Win2K, WinXP, Win2K3, WinVista, Mac, Mac OS X, Win3x, OS2, Unix, Linux
 
 =back
 
@@ -493,31 +644,47 @@ B<HTTP::Headers::UserAgent> module.
 The following methods are available, each returning a true or false value.  Some methods also
 test for the browser version, saving you from checking the version separately.
 
-  mosaic
-  netscape nav2 nav3 nav4 nav4up nav45 nav5 navgold
-  ie ie3 ie4 ie4up ie5 ie55
-  aol aol3 
+  netscape nav2 nav3 nav4 nav4up nav45 nav45up navgold nav6 nav6up
+  gecko 
+  mozilla
+  firefox 
+  safari
+  ie ie3 ie4 ie4up ie5 ie55 ie6
   neoplanet neoplanet2 
+  mosaic
+  aol aol3 aol4 aol5 aol6
   webtv
-  opera
-  lynx
+  opera opera3 opera4 opera5 opera6 opera7
+  lynx links
   emacs
   staroffice
   lotusnotes
   icab
   konqueror
   java
+  curl
+
+Netscape 6, even though its called six, in the userAgent string has version number 5.  The nav6 and nav6up methods correctly handle this quirk.
+The firefox text correctly detects the older-named versions of the browser (Phoenix, Firebird)
 
 =over
 
+
 =item browser_string()
 
 Returns one of the following strings, or undef.
 
 Netscape, MSIE, WebTV, AOL Browser, Opera, Mosaic, Lynx
 
+=item gecko_version() 
+
+If a Gecko rendering engine is used (as in Mozilla or Firebird), returns the version of the renderer (e.g. 1.3a, 1.7, 1.8)
+This might be more useful than the particular browser name or version when correcting for quirks in different versions of this rendering engine.
+If no Gecko browser is being used, or the version number can't be detected, returns undef.
+
 =back
 
+
 =head2 DETECTING OTHER DEVICES
 
 The following methods are available, each returning a true or false value.
@@ -527,6 +694,7 @@ The following methods are available, each returning a true or false value.
   iopener
   palm
   avantgo
+  blackberry
 
 =head2 DETECTING ROBOTS
 
@@ -555,14 +723,19 @@ exist on the Web.
 
 Lee Semel, lee@semel.net
 
+Thanks to Leonardo Herrera for additional contributions
 
 =head1 SEE ALSO
 
-perl(1).
+"The Ultimate JavaScript Client Sniffer, Version 3.0", B<http://www.mozilla.org/docs/web-developer/sniffer/browser_type.html>.
+
+"Browser ID (User-Agent) Strings" B<http://www.zytrax.com/tech/web/browser_ids.htm>
+
+perl(1), L<HTTP::Headers>, L<HTTP::Headers::UserAgent>.
 
 =head1 COPYRIGHT
 
-Copyright 1999-2000 Lee Semel.  All rights reserved.  This program is free software;
+Copyright 1999-2004 Lee Semel.  All rights reserved.  This program is free software;
 you can redistribute it and/or modify it under the same terms as Perl itself. 
 
 =cut
